/*
 * Copyright (c) 2015-2018 LAAS/CNRS
 * All rights reserved.
 *
 * Redistribution and use  in source  and binary  forms,  with or without
 * modification, are permitted provided that the following conditions are
 * met:
 *
 *   1. Redistributions of  source  code must retain the  above copyright
 *      notice and this list of conditions.
 *   2. Redistributions in binary form must reproduce the above copyright
 *      notice and  this list of  conditions in the  documentation and/or
 *      other materials provided with the distribution.
 *
 *					Anthony Mallet on Mon Aug 20 2018
 */
#ifndef H_MCCDAQFT_CODELS
#define H_MCCDAQFT_CODELS

#include "mccdaqft_c_types.h"

#include <aio.h>
#include <errno.h>
#include <inttypes.h>
#include <stdbool.h>
#include <string.h>

struct mccdaqft_log_s {
  struct aiocb req;
  char buffer[4096];
  bool pending, skipped;
  uint32_t decimation;
  size_t missed, total;

# define mccdaqft_logfmt	"%g "
# define mccdaqft_log_header                                         \
  "ts "                                                              \
  "fx fy fz  tx ty tz"
# define mccdaqft_log_line                                           \
  "%" PRIu64 ".%09d "                                                \
  mccdaqft_logfmt mccdaqft_logfmt mccdaqft_logfmt                    \
  mccdaqft_logfmt mccdaqft_logfmt mccdaqft_logfmt
};

static inline genom_event
mccdaqft_e_sys_error(const char *s, genom_context self)
{
  mccdaqft_e_sys_detail d;
  size_t l = 0;

  d.code = errno;
  if (s) {
    strncpy(d.what, s, sizeof(d.what) - 3);
    l = strlen(s);
    strcpy(d.what + l, ": ");
    l += 2;
  }
  strerror_r(d.code, d.what + l, sizeof(d.what) - l);
  return mccdaqft_e_sys(&d, self);
}

#ifdef __cplusplus
extern "C" {
#endif

  void	daqft_get_calibration(const or_t3d_pos *pos, const or_t3d_att *att,
                              const double calibration[36],
                              const double scale[6],
                              const or_t3d_pos *p, const or_t3d_att *q,
                              double G[36]);
  void	daqft_get_data(const double data[6],
                       const double G[36], const double b[6],
                       double w[6]);

#ifdef __cplusplus
}
#endif


#endif /* H_MCCDAQFT_CODELS */
